
extern "C"
{

#include <sys/types.h>
#include <sys/errno.h>
#include <sys/param.h>
//#include <sys/conf.h>
#include <sys/kernel.h>
#include <sys/module.h>

#if __FreeBSD_version >= 700000
#include <sys/systm.h>
#else
#include "systm.h"
#endif

#include <sys/bus.h>

#include <sys/selinfo.h>
#include <sys/taskqueue.h>

#include "dvbapi.h"

#include "devio.h"
#include "dvbif.h"
#include "util.h"

#include "dvbcore.h"
}

#include "tsdemux.h"
#include "fifodemux.h"
#include "mutex.h"
#include "fifo.h"

#include "dvbhal.h"

MODULE_VERSION(dvbhal, 1);

//
int
dvbControlFn(void *data, struct dvbapi *parm)
{
	struct dvbhal *hal = (struct dvbhal *)data;
	int res;

	if (hal == NULL)
		return -1;

	CTSDemux *tsdemux = hal->tsdemux;
	CFifoDemux *fifodemux = hal->fifodemux;
	CFifo *tsfifo = hal->tsfifo;
	
	switch (parm->code)
	{
	case IODMA: {
			tsfifo->Lock();
			parm->u.dma.fullness = (tsfifo->Size() % 5000000) / 100000;
			tsfifo->Unlock();

			res = 0;
			} break;

	case IODEMUX: {
			parm->u.demux.TSPkts = tsdemux->GetTSPkts();
			parm->u.demux.BadTSPkts = tsdemux->GetBadTSPkts();
			parm->u.demux.DisCont = tsdemux->GetDisCont();

			res = 0;
			} break;

	case IOGETFILTER: {

			unsigned char   filterNo;

			filterNo = parm->u.filter.filterNo;

			parm->u.filter.pid = tsdemux->GetPid(filterNo);
			parm->u.filter.bytes = tsdemux->GetBytes(filterNo);
			parm->u.filter.errors = tsdemux->GetErrors(filterNo);

			res = 0;
			} break;

	case IOADDFILTER: {

                        unsigned char   filterNo;

			tsdemux->AddFilter(filterNo,
					MPE_SECTION_FILTER,
					parm->u.filter.pid,
					&parm->u.filter.mac[0],
					&parm->u.filter.mask[0],
					188*3);

			res = 0;
			//TODO
			} break;

	case IODELFILTER: {

			unsigned char   filterNo;

			filterNo = parm->u.filterNo;

			tsdemux->DeleteFilter(filterNo);

			res = 0;
			//TODO
			} break;
	default:;

		res = -1;
	} //switch

	if (hal->devControlFn != NULL && hal->devdata != NULL)
	{
		if (hal->devControlFn(hal->devdata, parm) == -1)
			res = -1;
		else 
			res = 0;
	} 

	return res;
}

//
void
dvbDemuxNotify(struct dvbhal *hal, unsigned char filterType, unsigned char filterNo,
				unsigned char *data, unsigned int len, unsigned char flag)
{
	unsigned short seclen;
	unsigned short datalen;

	if (filterType == MPE_SECTION_FILTER || filterType == MULTI_MPE_FILTER)
	{
		seclen = sec_length(data);

		if (data[5] & 2)
		{
			if (data[18] == 8 && data[19] == 0)
				datalen = (data[22] << 8) + data[23] + 8;
			else
				datalen = seclen - 16;
		} else
		{
			datalen = (data[14] << 8) + data[15];
		}

		if (data[0] != 0x3E)
		{
			hal->NotMPE ++;
		}

		if ( !(data[5] & 0x3C) )
		{
			if ( (data[1] & 0x80) && !SectionCrcCheck(data, seclen))
			{
				hal->BadSecCRC ++;
				goto endcheck;
			}

			if ( !(data[1] & 0x80) )
			{
				if ( *(unsigned int *)&data[seclen - 4] == 0)
				{
					hal->NoSecCSum ++;
					goto endcheck;
				}

				if ( !SectionCSumCheck(data, seclen))
				{
					hal->BadSecCSum ++;
					goto endcheck;
				}
			} else
			{
				datalen += 16;
				if (seclen < datalen)
				{
					hal->SecBadL ++;
					goto endcheck;
				}

				if (seclen > len || datalen > len)
				{
					hal->FltBadL ++;
					goto endcheck;
				}
			}

		} else
		{
			hal->Scrmbl ++;

			if (data[5] & 2 && (data[18] != 8 || data[19] != 0))
			{
				hal->NotIP ++;
			}
		}
	}

endcheck:
#if 0
	if (filterType != MPE_SECTION_FILTER && filterType != MULTI_MPE_FILTER)
	{
		len -= 28;
	} else
	{
		len -= 4;       //?checksum
	}
#endif
	/*
	TODO
	*/

	dvbDemuxMPE(hal, &data[0], len);
}

//
void
dvbReleaseObject(struct dvbhal *hal, int level)
{
	switch (level)
	{

	case 10: {
			if (hal->fifodemux != NULL)
			{
				delete hal->fifodemux;
				hal->fifodemux = 0;//NULL
			}
		}

	case 9: {
			if (hal->tsfifo != NULL)
			{
				delete hal->tsfifo;
				hal->tsfifo = 0;//NULL;
			}
		}

	case 8: {
			if (hal->tsdemux != NULL)
			{
				delete hal->tsdemux;
				hal->tsdemux = 0;//NULL;
			}
		}
	} //switch
}

//
int
dvbCreateObject(struct dvbhal *hal)
{
	hal->tsdemux = new CTSDemux(hal, DEMUXFN(&dvbDemuxNotify));
	if (hal->tsdemux == NULL)
	{
		dvbReleaseObject(hal, 7);
		return 8;
	}

	hal->tsfifo = new CFifo(5000000);
	if (hal->tsfifo == NULL || hal->tsfifo->IsOK() == 0)
	{
		dvbReleaseObject(hal, 8);
		return 9;
	}

	hal->fifodemux = new CFifoDemux(hal->tsfifo, hal->tsdemux);
	if (hal->fifodemux == NULL || hal->fifodemux->IsOK() == 0)
	{
		dvbReleaseObject(hal, 9);
		return 10;
	}

	return 0;
}

//
void *
dvbCreate(device_t dev, unsigned char *MacAddress,
		void *devdata, devControlFn_t devControlFn,
		void *netdata, netEventFn_t netEvent)
{
	struct dvbhal *hal;
	int res;

	hal = new dvbhal;
	if (hal == NULL)
		return NULL;

	bzero(hal, sizeof(*hal));

	if (dev == NULL)
		return NULL;

	hal->devdata = netdata;
	hal->devControlFn = devControlFn;

	hal->dvbdev = (struct dvbdevice *)device_get_softc(dev);

	dvb_init(hal->dvbdev, dev, hal, dvbControlFn);

	dvb_ifattach(&hal->dvbif, MacAddress, netdata, netEvent);

	res = dvbCreateObject(hal);
	if (res != 0)
	{
		switch (res)
		{
		case 8:
		device_printf(dev, "Error: Failed to create tsdemux!\n");
		break;

		case 9:
		device_printf(dev, "Error: Failed to create fifo!\n");
		break;

		case 10:
		device_printf(dev, "Error: Failed to create fifodemux!\n");
		break;

		default:
		device_printf(dev, "Error: Failed unknown error!\n");

		}

		dvbRelease(hal);
		return NULL;
	}

	device_printf(dev, "Info: dvbdevice create!\n");
	return hal;
}

//
int
dvbRelease(void *data)
{
	struct dvbhal *hal = (struct dvbhal *)data;

	if (hal == NULL)
		return -1;

	dvbReleaseObject(hal, 12);

	dvb_ifdetach(&(hal->dvbif));

	dvb_free(hal->dvbdev);

	delete hal;

	return 0;
}

//
int
dvbDemuxMPE(void *data, unsigned char *buf, unsigned int len)
{
	struct dvbhal *hal = (struct dvbhal *)data;

	if (hal == NULL)
		return -1;

	CTSDemux *tsdemux = hal->tsdemux;
	CFifoDemux *fifodemux = hal->fifodemux;
	CFifo *tsfifo = hal->tsfifo;

	dvb_input(&hal->dvbif, buf, len);
}

//
int
dvbDemuxTS(void *data, unsigned char *buf, unsigned int len)
{
	struct dvbhal *hal = (struct dvbhal *)data;

	if (hal == NULL)
		return -1;

	CTSDemux *tsdemux = hal->tsdemux;
	CFifoDemux *fifodemux = hal->fifodemux;
	CFifo *tsfifo = hal->tsfifo;
	int res;
#if 0
	tsdemux->DemuxTS(&buf[0], len/188);
#else
	tsfifo->Lock();
	res = tsfifo->Put(&buf[0], len);
	tsfifo->Unlock();

	if (res == 0 || res != len)
	{
		tsfifo->Reset();
		res = 0;
	}

	fifodemux->Schedule();
#endif
	return res;
}

//
int
dvbResetTS(void *data)
{
	struct dvbhal *hal = (struct dvbhal *)data;

	if (hal == NULL)
		return -1;

	CTSDemux *tsdemux = hal->tsdemux;
	CFifoDemux *fifodemux = hal->fifodemux;
	CFifo *tsfifo = hal->tsfifo;

	tsfifo->Reset();
	tsdemux->ResetStat();

	//tsdemux->EnablePIDCount(0);
	//tsdemux->EnablePIDCount(1);
	
	return 0;
}

