
/*
 *      writed by kir http://paradox.org.ua
 */

#include <sys/types.h>
#include <sys/errno.h>
#include <sys/param.h>
#include <sys/conf.h>
#include <sys/kernel.h>
#include <sys/bus.h>

#include <sys/selinfo.h>

#include "dvbapi.h"

#include "devio.h"

static int dev_num = 0;

static d_open_t         dvb_open;
static d_close_t        dvb_close;
static d_read_t         dvb_read;
static d_write_t        dvb_write;
static d_ioctl_t        dvb_ioctl;
static d_mmap_t         dvb_mmap;
static d_poll_t         dvb_poll;

static struct cdevsw dvb_cdevsw = {
	.d_version =    D_VERSION,
	.d_flags =      D_NEEDGIANT,
	.d_open =       dvb_open,
	.d_close =      dvb_close,
	.d_read =       dvb_read,
	.d_write =      dvb_write,
	.d_ioctl =      dvb_ioctl,
	.d_poll =       dvb_poll,
	.d_mmap =       dvb_mmap,
	.d_name =       "dvb"
};

void
dvb_init(struct dvbdevice *dvbdev, device_t dev, void *devdata, devControlFn_t devControlFn)
{
	//knlist_init(&dvbdev->kq_sel.si_note, NULL, NULL, NULL, NULL);

	dvbdev->devdata = devdata;
	dvbdev->devControlFn = devControlFn;

	dvbdev->dev = make_dev(&dvb_cdevsw,
				dev_num,
				UID_ROOT,
				GID_WHEEL,
				0600,
				"dvb%d", dev_num);

	dvbdev->dev->si_drv1 = dev;

	dev_num ++;
}

void
dvb_free(struct dvbdevice *dvbdev)
{
	dev_num --;

	destroy_dev(dvbdev->dev);

	//knlist_destroy(&dvbdev->kq_sel.si_note);
}


static int
dvb_open(struct cdev *dev, int flags, int fmt, struct thread *td)
{
	struct dvbdevice *dvbdev;
	int unit;
	int err;

	dvbdev = device_get_softc(dev->si_drv1);
	if (dvbdev == NULL)
	{
		return ENXIO;
	}

	device_busy(dev->si_drv1);

	return 0;
}

static int
dvb_close(struct cdev *dev, int flags, int fmt, struct thread *td)
{
	struct dvbdevice *dvbdev;
	int unit;
	int err;

	dvbdev = device_get_softc(dev->si_drv1);
	if (dvbdev == NULL)
	{
		return ENXIO;
	}

	device_unbusy(dev->si_drv1);

	return 0;
}

static int
dvb_read(struct cdev *dev, struct uio *uio, int ioflag)
{
	return ENXIO;
}

static int
dvb_write(struct cdev *dev, struct uio *uio, int ioflag)
{
	return ENXIO;
}

static int
dvb_ioctl(struct cdev *dev, unsigned long cmd, caddr_t arg, int flag, struct thread *td)
{
	struct dvbdevice *dvbdev;
	int unit;
	int err;

	dvbdev = device_get_softc(dev->si_drv1);
	if (dvbdev == NULL)
	{
		return ENXIO;
	}

	err = 0;

	switch (cmd)
	{
	case DVBAPI:
		{
			struct dvbapi *parm = (struct dvbapi *)arg;

			if (dvbdev->devControlFn != NULL && dvbdev->devdata != NULL)
			{
				if (dvbdev->devControlFn(dvbdev->devdata, parm) == -1)
				{
					err = EINVAL;
				} else
				{
					err = 0;
				}
			} else
				err = EINVAL;
		}
			break;

	default:
		printf("%s: unsupported ioctl %08x!\n", __FUNCTION__, (unsigned long)cmd);
		err = EINVAL;
	}

	return err;
}

static int
dvb_mmap(struct cdev *dev, vm_offset_t offset, vm_paddr_t *paddr, int nprot)
{
	return ENXIO;
}

static int
dvb_poll(struct cdev *dev, int events, struct thread *tp)
{
	return ENXIO;
}

