
/*
 *      writed by kir http://paradox.org.ua
 */

extern "C"
{
	#include <sys/types.h>
	#include <sys/param.h>
#if __FreeBSD_version >= 700000
	#include <sys/systm.h>
#else
	#include "systm.h"
#endif
	//#include <sys/_lock.h>
	//#include <sys/_mutex.h>

}

#include "mutex.h"
#include "fifo.h"

CFifo::CFifo(unsigned int size)
{
	m_pBuf = 0;//NULL;
	m_nSize = 0;

	m_dwReadIdx = 0;
	m_dwWriteIdx = 0;

	Init(size);
}

CFifo::~CFifo()
{
	if (m_pBuf != NULL)
	{
		delete []m_pBuf;
		m_pBuf = 0;//NULL;
	}
}

int
CFifo::Init(unsigned int size)
{

	unsigned char *buffer;

	buffer = new unsigned char[size];
	if (buffer == NULL)
	{
		return 0;
	}

	m_pBuf = buffer;

	m_dwReadIdx = 0;
	m_dwWriteIdx = 0;

	m_nSize = size;

	return 1;
}

int
CFifo::IsOK(void)
{
	if (m_pBuf == NULL)
		return 0;

	return 1;
}

void
CFifo::Reset(void)
{
	m_dwReadIdx = 0;
	m_dwWriteIdx = 0;
}

unsigned int
CFifo::Size(void)
{
	unsigned int resSize;

	resSize = 0;

	if (m_pBuf == NULL)
	{
		return 0;
	}

	if (m_dwReadIdx == m_dwWriteIdx)
	{
		return 0;
	}

	if (m_dwReadIdx < m_dwWriteIdx)
	{
		resSize = m_dwWriteIdx - m_dwReadIdx;
	} else
	{
		resSize = (m_nSize - m_dwReadIdx) + m_dwWriteIdx;
	}

	return resSize;
}

unsigned int
CFifo::Put(unsigned char *buffer, unsigned int size)
{
	unsigned int resSize;

	unsigned int TotalSize;
	unsigned int FreeSize;
	unsigned int AvailSize;

	if (m_pBuf == NULL)
	{
		return 0;
	}

	if (m_dwReadIdx > m_dwWriteIdx)
	{
		AvailSize = (m_dwReadIdx - m_dwWriteIdx) - 1;
		FreeSize = 0;
	} else
	{
		AvailSize = m_nSize - m_dwWriteIdx;
		if (m_dwReadIdx == 0)
		{
			AvailSize --;
			FreeSize = 0;
		} else
		{
			FreeSize = m_dwReadIdx - 1;
		}
	}

	TotalSize = AvailSize + FreeSize;

		resSize = size;
	if (TotalSize < size)
		resSize = TotalSize;

//
	unsigned int AvailWrite;

		AvailWrite = resSize;
	if (AvailSize < AvailWrite)
		AvailWrite = AvailSize;

	if (AvailWrite > 0)
	{
		memcpy(m_pBuf + m_dwWriteIdx, buffer, AvailWrite);
		m_dwWriteIdx += AvailWrite;
		buffer += AvailWrite;
	}
//
	unsigned int FreeWrite;

		FreeWrite = resSize - AvailWrite;
	if (FreeSize < FreeWrite)
		FreeWrite = FreeSize;

	if (FreeWrite > 0)
	{
		memcpy(m_pBuf, buffer, FreeWrite);
		m_dwWriteIdx = FreeWrite;
	}
//
	if (m_dwWriteIdx == m_nSize)
	{
		m_dwWriteIdx = 0;
	}

	return resSize;
}

unsigned int
CFifo::Get(unsigned char *buffer, unsigned int size)
{
	unsigned int resSize;

	unsigned int TotalSize;
	unsigned int FreeSize;
	unsigned int AvailSize;

	if (m_pBuf == NULL)
	{
		return 0;
	}

	if (m_dwReadIdx == m_dwWriteIdx)
	{
		return 0;
	}

	if (m_dwReadIdx < m_dwWriteIdx)
	{
		AvailSize = m_dwWriteIdx - m_dwReadIdx;
		FreeSize = 0;
	} else
	{
		AvailSize = m_nSize - m_dwReadIdx;
		FreeSize = m_dwWriteIdx;
	}

	TotalSize = AvailSize + FreeSize;

		resSize = size;
	if (TotalSize < size)
		resSize = TotalSize;

//
	unsigned int AvailRead;

		AvailRead = resSize;
	if (AvailSize < AvailRead)
		AvailRead = AvailSize;

	if (AvailRead > 0)
	{
		memcpy(buffer, m_pBuf + m_dwReadIdx, AvailRead);
		m_dwReadIdx += AvailRead;
		buffer += AvailRead;
	}
//
	unsigned int FreeRead;

		FreeRead = resSize - AvailRead;
	if (FreeSize < FreeRead)
		FreeRead = FreeSize;

	if (FreeRead > 0)
	{
		memcpy(buffer, m_pBuf, FreeRead);
		m_dwReadIdx = FreeRead;
	}
//
	if (m_dwReadIdx == m_nSize)
	{
		m_dwReadIdx = 0;
	}

	return resSize;
}

void
CFifo::Lock(void)
{
	//mutex.Lock();
}

void
CFifo::Unlock(void)
{
	//mutex.Unlock();
}

