
#include "i2c.h"
#include "eeprom.h"

CEEPROM::CEEPROM(class CFLexI2C *flexi2c):m_pFlexI2c(flexi2c)
{
}

CEEPROM::~CEEPROM()
{
}

int
CEEPROM::read(unsigned short addr, unsigned char *buf, unsigned short len)
{
	int res;
	struct _I2CBUS  bus;

	bus.device = 0x20000000;
	bus.unk4 = ((addr & 0x300) | 0x5000) >> 8;
	bus.retr = 500;
	bus.unkC = 0;
	bus.unkD = 0;
	bus.unkE = 0;
	bus.unkF = 0;
	bus.eeprom = 1;
	bus.unk14 = 0;
	bus.unk18 = 0;
	bus.wait = 5;

	res = m_pFlexI2c->BusRead(&bus, addr, buf, len);

	DELAY(5);

	return res;
}

int
CEEPROM::write(unsigned short addr, unsigned char *buf, unsigned short len)
{
	int res;
	struct _I2CBUS  bus;

	bus.device = 0x20000000;
	bus.unk4 = ((addr & 0x300) | 0x5000) >> 8;
	bus.retr = 500;
	bus.unkC = 0;
	bus.unkD = 0;
	bus.unkE = 0;
	bus.unkF = 0;
	bus.eeprom = 1;
	bus.unk14 = 0;
	bus.unk18 = 0;
	bus.wait = 5;

	res = m_pFlexI2c->BusWrite(&bus, addr, buf, len);

	DELAY(5);

	return res;
}

unsigned char
calc_LRC(unsigned char *buf, int len)
{
	int i;
	unsigned char sum;

	sum = 0;
	for (i = 0; i < len; i++)
		sum = sum ^ buf[i];
	return sum;
}

int
CEEPROM::LRC_read(unsigned int addr, unsigned int len, unsigned char *buf, int retr)
{
	int i;

	for (i = 0; i < retr; i++)
	{
		if (read(sc, addr, buf, len) == len)
		{
			if (calc_LRC(buf, len-1) == buf[len-1])
				return 1;
		}
	}
	return 0;
}

int
CEEPROM::LRC_write(unsigned int addr, unsigned int len, unsigned char *wbuf, unsigned char *rbuf, int retr)
{
	int i;

	for (i = 0; i < retr; i++)
	{
		if (write(sc, addr, wbuf, len) == len)
		{
			if (LRC_read(sc, addr, len, rbuf, retr) == 1)
				return 1;
		}
	}
	return 0;
}

int
CEEPROM::getMacAddr(char type, unsigned char *mac)
{
	int res;
	unsigned char tmp[8];

	if (LRC_read(0x3F8, 8, tmp, 4) != 0)
	{
		if (type != 0)
		{
			mac[0] = tmp[0];
			mac[1] = tmp[1];
			mac[2] = tmp[2];
			mac[3] = 0xFE;
			mac[4] = 0xFF;
			mac[5] = tmp[3];
			mac[6] = tmp[4];
			mac[7] = tmp[5];
		} else {
			mac[0] = tmp[0];
			mac[1] = tmp[1];
			mac[2] = tmp[2];
			mac[3] = tmp[3];
			mac[4] = tmp[4];
			mac[5] = tmp[5];
		}
		res = 1;
	} else {
		if (type != 0) memset(mac, 0, 8);
			else   memset(mac, 0, 6);
		res = 0;
	}
	return res;
}

int
CEEPROM::setMacAddr(char type, unsigned char *mac)
{
	int res;
	unsigned char tmp[8];

	if (type != 0)
	{
		tmp[0] = mac[0];
		tmp[1] = mac[1];
		tmp[2] = mac[2];
		tmp[3] = mac[5];
		tmp[4] = mac[6];
		tmp[5] = mac[7];
	} else {
		tmp[0] = mac[0];
		tmp[1] = mac[1];
		tmp[2] = mac[2];
		tmp[3] = mac[3];
		tmp[4] = mac[4];
		tmp[5] = mac[5];
	}

	tmp[6] = 0;
	tmp[7] = calc_LRC(tmp, 7);

	if (write(0x3F8, tmp, 8) == 8)
		res = 1;
	res = 0;

	return res;
}
