
#include "i2c.h"

CFlexI2C::CFlexI2C(class CSpci *pci):m_pSpci(pci)
{
}

CFlexI2C::~CFLexI2C()
{
}

int
CFlexI2C::I2cRW(unsigned int device, unsigned int bus, unsigned int dir,
	unsigned int addr, unsigned char *buf, unsigned int len,
	unsigned int unkC, unsigned int retr, unsigned int unkD,
	unsigned int unkE, unsigned int unkF, unsigned int unk14,
	unsigned int eeprom, unsigned int bank)
{
	unsigned int cmd;
	unsigned int val;
	unsigned int bankaddr;
	unsigned int bankval;
	int i;
	int r;
	int res;
	int state;
	int done;

	res = 0;
	bankval = 0;

	if (bank)
	{
		if (device == 0x20000000)
			bankaddr = 0x10C;
		else
		if (device == 0x30000000)
			bankaddr = 0x110;
		else
			bankaddr = 0x108;

		bankval = m_pSpci->GetReg(bankaddr);

		m_pSpci->SetReg(bankaddr, bank);
	}

	state = unkD ? 3 : 0;

	if (addr > 0xFF && eeprom == 1)
	{
		bus |= ((addr >> 8) & 3);
	}

	cmd = device | ((len - 1) << 26) | (0 << 16) | (addr << 8) | bus;
	cmd |= 0x01000000;

	if (dir != 0)
	{
		cmd |= 0x02000000;
	} else
	{
		cmd |= (buf[0] << 16);

		if (len > 1)
		{
			val = 0;
			for (i = len; i > 1; i--)
			{
				val <<= 8;
				val |= buf[i-1];
			}

			m_pSpci->SetReg(0x104, val);
		}
	}

	done = 0;

	for (r = 0; r < 100000; r++)
	{
		if (done != 0)
			break;

		switch (state)
		{
		case 3:
			m_pSpci->SetReg(0x100, 0);
			m_pSpci->SetReg(0x100, device | ((unkE | unkD | 0x100) << 16) | (unkF << 8) | unk14);
			state = 5;
			break;

		case 5:
			val = m_pSpci->GetReg(0x100);

			if (val & 0x40000000)
			{
				if (retr--)
					state = 3;
				else
					done = 1;
			} else
			{
				if ((val & 0x81000000) == 0x80000000)
					state = 0;
			}
			break;

		case 0:
			m_pSpci->SetReg(0x100, 0);
			m_pSpci->SetReg(0x100, cmd);

			if ((unkC != 0) && (cmd & 0x2000000))
				state = 1;
			else
				state = 2;
			break;

		case 1:
			val = m_pSpci->GetReg(0x100);

			if (! (val &  0x40000000) )
				break;

			m_pSpci->SetReg(0x100, 0);

			if (unkD == 0)
			{
				m_pSpci->SetReg(0x100, cmd | 0x40000000);
				state = 2;
			} else
			{
				m_pSpci->SetReg(0x100, device | ((unkE | unkD | 0x100) << 16) | (unkF << 8) | unk14);
				state = 6;
			}
			break;

		case 6:
			val = m_pSpci->GetReg(0x100);

			if (val & 0x40000000)
			{
				if (retr --)
					state = 1;
				else
					done = 1;
			} else
			{
				if ((val & 0x81000000) == 0x80000000)
				{
					m_pSpci->SetReg(0x100, 0);
					m_pSpci->SetReg(0x100, cmd | 0x40000000);
					state = 2;
				}
			}
			break;

		case 2:
			val = m_pSpci->GetReg(0x100);

			if (val & 0x40000000)
			{
				if (retr --)
				{
					state = unkD ? 3 : 0;
				} else
				{
					if (dir != 0 && len > 0)
						memset(&buf[0], 0xFF, len);
					done = 1;
				}
			} else
			{
				if ((val & 0x81000000) != 0x80000000)
					break;

				if (dir != 0)
				{
					buf[0] = (val >> 16) & 0xFF;

					if (len > 1)
					{
						val = m_pSpci->GetReg(0x104);

						for (i = 1; i < len; i++)
						{
							buf[i] = val & 0xFF;
							val >>= 8;
						}
					}
				}
				done = 1;
				res = 1;
			}
			break;

		} //switch
	} //for

	if (bankval)
	{
		m_pSpci->SetReg(bankaddr, bankval);
	}

	return res;
}

int
CFlexI2C::Read4(unsigned int device, unsigned int bus,
	unsigned int addr, unsigned char *buf, unsigned int len,
	unsigned int unkC, unsigned int retr, unsigned int unkD,
	unsigned int unkE, unsigned int unkF, unsigned int unk14,
	unsigned int eeprom, unsigned int bank)
{
	return I2cRW(device, bus, 1,
			addr, buf, len,
			unkC, retr, unkD,
			unkE, unkF, unk14,
			eeprom, bank);
}

int
CFLexI2C::Write4(unsigned int device, unsigned int bus,
	unsigned int addr, unsigned char *buf, unsigned int len,
	unsigned int unkC, unsigned int retr, unsigned int unkD,
	unsigned int unkE, unsigned int unkF, unsigned int unk14,
	unsigned int eeprom, unsigned int bank)
{
	return I2cRW(device, bus, 0,
			addr, buf, len,
			unkC, retr, unkD,
			unkE, unkF, unk14,
			eeprom, bank);
}

unsigned int
CFlexI2C::BusRead(struct _I2CBUS *bus, unsigned int addr,
	unsigned char *buf, unsigned int len)
{
	unsigned int bytes2transfer;
	unsigned char *start;

	if (bus == NULL)
		return 0;

	start = buf;
	while (len != 0)
	{
		bytes2transfer = len;

		if (bytes2transfer > 4)
			bytes2transfer = 4;

		if (Read4(bus->device,
				bus->unk4,
				addr,
				buf,
				bytes2transfer,
				bus->unkC,
				bus->retr,
				bus->unkD,
				bus->unkE,
				bus->unkF,
				bus->unk14,
				bus->eeprom,
				bus->unk18) == 0)
			return (buf - start);

		buf += bytes2transfer;
		addr += bytes2transfer;
		len -= bytes2transfer;
	}
	return (buf - start);
}

unsigned int
CFlexI2C::BusWrite(struct _I2CBUS *bus, unsigned int addr,
	unsigned char *buf, unsigned int len)
{
	unsigned int bytes2transfer;
	unsigned char *start;

	if (bus == NULL)
		return 0;

	start = buf;
	while (len != 0)
	{
		bytes2transfer = len;

		if (bytes2transfer > 4)
			bytes2transfer = 4;

		if (Write4(bus->device,
				bus->unk4,
				addr,
				buf,
				bytes2transfer,
				bus->unkC,
				bus->retr,
				bus->unkD,
				bus->unkE,
				bus->unkF,
				bus->unk14,
				bus->eeprom,
				bus->unk18) == 0)
			return (buf - start);

		buf += bytes2transfer;
		addr += bytes2transfer;
		len -= bytes2transfer;
	}
	return (buf - start);
}

unsigned int
CFlexI2C::Read(unsigned int device, unsigned int bus,
	unsigned int addr, unsigned char *buf, unsigned int len)
{
	unsigned int bytes2transfer;
	unsigned char *start;

	start = buf;
	while (len != 0)
	{
		bytes2transfer = len;

		if (bytes2transfer > 4)
			bytes2transfer = 4;

		if (Read4(device,
				bus,
				addr,
				buf,
				bytes2transfer,
				0,
				1,
				0,
				0,
				0,
				0,
				0,
				0) == 0)
			return (buf - start);

		buf += bytes2transfer;
		addr += bytes2transfer;
		len -= bytes2transfer;
	}
	return (buf - start);
}

unsigned int
CFLexI2C::Write(unsigned int device, unsigned int bus,
	unsigned int addr, unsigned char *buf, unsigned int len)
{
	unsigned int bytes2transfer;
	unsigned char *start;

	start = buf;
	while (len != 0)
	{
		bytes2transfer = len;

		if (bytes2transfer > 4)
			bytes2transfer = 4;

		if (Write4(device,
				bus,
				addr,
				buf,
				bytes2transfer,
				0,
				1,
				0,
				0,
				0,
				0,
				0,
				0) == 0)
			return (buf - start);

		buf += bytes2transfer;
		addr += bytes2transfer;
		len -= bytes2transfer;
	}
	return (buf - start);
}

